#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Created by iFantastic on 2020/4/16


class ErrorMessage:

    reques_parameter_error = "Request Parameter Error..."
    server_inner_error = "Server Inner Error..."

